<?php

/*
Plugin Name: Delete Duplicate Posts Pro
Plugin Script: delete-duplicate-posts.php
Plugin URI: https://cleverplugins.com
Description: Remove duplicate blogposts on your blog! Searches and removes duplicate posts and their post meta tags. You can delete posts, pages and other Custom Post Types enabled on your website.
Version: 4.9.4
Update URI: https://api.freemius.com
Author: cleverplugins.com
Author URI: https://cleverplugins.com
Min WP Version: 4.7
Max WP Version: 6.4.2
Text Domain: delete-duplicate-posts
Domain Path: /languages
*/
namespace DeleteDuplicatePosts;

if ( !defined( 'ABSPATH' ) ) {
    exit;
}

if ( function_exists( 'ddp_fs' ) ) {
    ddp_fs()->set_basename( true, __FILE__ );
} else {
    // DO NOT REMOVE THIS IF, IT IS ESSENTIAL FOR THE `function_exists` CALL ABOVE TO PROPERLY WORK.
    
    if ( !function_exists( 'ddp_fs' ) ) {
        // Create a helper function for easy SDK access.
        function ddp_fs()
        {
            global  $ddp_fs ;
            
            if ( !isset( $ddp_fs ) ) {
                // Activate multisite network integration.
                if ( !defined( 'WP_FS__PRODUCT_925_MULTISITE' ) ) {
                    define( 'WP_FS__PRODUCT_925_MULTISITE', true );
                }
                require_once __DIR__ . '/freemius/start.php';
                define( 'CP_DDP_FREEMIUS_STATE', 'cp_ddp_freemius_state' );
                $cp_ddp_freemius_state = get_site_option( CP_DDP_FREEMIUS_STATE, 'anonymous' );
                $is_anonymous = 'anonymous' === $cp_ddp_freemius_state || 'skipped' === $cp_ddp_freemius_state;
                $is_premium = false;
                $is_anonymous = ( $is_premium ? false : $is_anonymous );
                $ddp_fs = fs_dynamic_init( array(
                    'id'             => '925',
                    'slug'           => 'delete-duplicate-posts',
                    'type'           => 'plugin',
                    'public_key'     => 'pk_0af9f9e83f00e23728a55430a57dd',
                    'is_premium'     => true,
                    'premium_suffix' => 'Pro',
                    'has_addons'     => false,
                    'has_paid_plans' => true,
                    'anonymous_mode' => $is_anonymous,
                    'menu'           => array(
                    'slug'        => 'delete-duplicate-posts.php',
                    'first-path'  => 'tools.php?page=delete-duplicate-posts',
                    'support'     => false,
                    'affiliation' => false,
                    'parent'      => array(
                    'slug' => 'tools.php',
                ),
                ),
                    'is_live'        => true,
                ) );
            }
            
            return $ddp_fs;
        }
        
        ddp_fs();
        do_action( 'ddp_fs_loaded' );
    }
    
    ddp_fs()->add_action( 'after_uninstall', 'ddp_fs_uninstall_cleanup' );
    /**
     * Cleans up when uninstalling
     *
     * @author   Lars Koudal
     * @since    v0.0.1
     * @version  v1.0.0  Tuesday, January 12th, 2021.
     * @return   void
     */
    function ddp_fs_uninstall_cleanup()
    {
        global  $wpdb ;
        $wpdb->query( $wpdb->prepare( 'DROP TABLE IF EXISTS %s', $wpdb->prefix . 'ddp_log' ) );
        $wpdb->query( $wpdb->prepare( 'DROP TABLE IF EXISTS %s', $wpdb->prefix . 'ddp_redirects' ) );
        delete_option( 'ddp_deleted_duplicates' );
        delete_option( 'delete_duplicate_posts_options_v4' );
        delete_option( 'cp_ddp_freemius_state' );
    }
    
    require plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';
}

add_action( 'admin_init', array( 'PAnD', 'init' ) );

if ( !class_exists( __NAMESPACE__ . '\\Delete_Duplicate_Posts' ) ) {
    class Delete_Duplicate_Posts
    {
        public static  $options_name = 'delete_duplicate_posts_options_v4' ;
        public  $localization_domain = 'delete-duplicate-posts' ;
        public static  $options = array() ;
        public function __construct()
        {
            // Adds extra permissions to Freemius
            if ( function_exists( 'ddp_fs' ) ) {
                ddp_fs()->add_filter( 'permission_list', array( __CLASS__, 'add_freemius_extra_permission' ) );
            }
            global  $ddp_fs ;
            $locale = get_locale();
            $mo = plugin_dir_path( __FILE__ ) . '/languages/delete-duplicate-posts-' . $locale . '.mo';
            load_plugin_textdomain( 'delete-duplicate-posts', false, __DIR__ . '/languages/' );
            add_action(
                'admin_head',
                array( __CLASS__, 'set_custom_help_content' ),
                1,
                2
            );
            self::get_options();
            add_action( 'wp_ajax_ddp_get_loglines', array( __CLASS__, 'return_loglines_ajax' ) );
            add_action( 'wp_ajax_ddp_get_duplicates', array( __CLASS__, 'return_duplicates_ajax' ) );
            if ( $ddp_fs->is__premium_only() ) {
                
                if ( $ddp_fs->can_use_premium_code() ) {
                    add_action( 'wp_ajax_ddp_get_redirects', array( __CLASS__, 'return_redirects_ajax__premium_only' ) );
                    add_action( 'wp_ajax_ddp_delete_redirects', array( __CLASS__, 'delete_redirects_ajax__premium_only' ) );
                }
            
            }
            add_action( 'wp_ajax_ddp_delete_duplicates', array( __CLASS__, 'delete_duplicates_ajax' ) );
            add_action( 'wp_ajax_cp_ddp_freemius_opt_in', array( __CLASS__, 'cp_ddp_fs_opt_in' ) );
            // loads admin notices
            add_action( 'admin_init', array( 'PAnD', 'init' ) );
            add_action( 'admin_menu', array( $this, 'admin_menu_link' ) );
            add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
            add_action(
                'wp_insert_site',
                array( $this, 'on_create_blog' ),
                99999,
                1
            );
            add_filter( 'wpmu_drop_tables', array( $this, 'on_delete_blog' ) );
            register_activation_hook( __FILE__, array( $this, 'install' ) );
            add_action( 'ddp_cron', array( $this, 'cleandupes' ) );
            add_action( 'cron_schedules', array( $this, 'add_cron_intervals' ) );
            add_action( 'admin_notices', array( $this, 'ddp_action_admin_notices' ) );
            if ( $ddp_fs->is__premium_only() ) {
                add_action( 'template_redirect', array( &$this, 'do_redirects__premium_only' ) );
            }
        }
        
        /**
         * Checks if there are any redirects
         *
         * @author  Unknown
         * @since   v0.0.1
         * @version v1.0.0  Sunday, July 17th, 2022.
         * @access  private static
         * @global
         * @return  void
         */
        public static function do_redirects__premium_only()
        {
            global  $wpdb ;
            $options = self::get_options();
            if ( !$options['ddp_redirects'] ) {
                return;
            }
            
            if ( isset( $_SERVER['HTTPS'] ) && ('on' === $_SERVER['HTTPS'] || 1 === $_SERVER['HTTPS']) || isset( $_SERVER['HTTP_X_FORWARDED_PROTO'] ) && 'https' === $_SERVER['HTTP_X_FORWARDED_PROTO'] ) {
                $protocol = 'https://';
            } else {
                $protocol = 'http://';
            }
            
            $currenturl = $protocol . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
            $currenturl_relative = wp_make_link_relative( $currenturl );
            $table_name = $wpdb->prefix . 'ddp_redirects';
            $targeturl = $wpdb->get_var( $wpdb->prepare( "SELECT targeturl FROM {$table_name}\n\t\t\t\t\t\t\tWHERE inurl = %s\n\t\t\t\t\t\t\tLIMIT 1\n\t\t\t\t\t\t\t", $currenturl_relative ) );
            
            if ( $targeturl ) {
                if ( $options['ddp_debug'] ) {
                    // Translators: %1 = the current URL. %2 = the target URL for the redirect.
                    self::log( sprintf( __( 'DEBUG: Redirected from %1$s to %2$s ', 'delete-duplicate-posts' ), $currenturl_relative, $targeturl ) );
                }
                wp_safe_redirect( $targeturl, 301, 'Delete Duplicate Posts' );
                exit;
            }
        
        }
        
        /**
         * delete_redirects_ajax__premium_only.
         *
         * @author	Unknown
         * @since	v0.0.1
         * @version	v1.0.0	Saturday, December 2nd, 2023.
         * @access	public static
         * @param	boolean	$return_data	Default: false
         * @return	void
         */
        public static function delete_redirects_ajax__premium_only( $return_data = false )
        {
            check_ajax_referer( 'cp_ddp_return_duplicates', true );
            
            if ( !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( 'You do not have sufficient permissions to perform this action.' );
                return;
            }
            
            $checked_posts = array();
            
            if ( isset( $_POST['checked_posts'] ) && is_array( $_POST['checked_posts'] ) ) {
                global  $wpdb ;
                $table_name = $wpdb->prefix . 'ddp_redirects';
                $count = 0;
                foreach ( $_POST['checked_posts'] as $cp ) {
                    $id = intval( $cp['ID'] );
                    $wpdb->delete( $table_name, array(
                        'ID' => $id,
                    ), array( '%d' ) );
                    $count++;
                }
            }
            
            self::log( sprintf( __( 'Deleted %1$s redirects.', 'delete-duplicate-posts' ), $count ) );
            wp_send_json_success();
        }
        
        /**
         * Ajax callback to handle freemius opt in/out.
         *
         * @author   Lars Koudal
         * @since    v0.0.1
         * @version  v1.0.0  Tuesday, January 12th, 2021.
         * @access   public static
         * @return   void
         */
        public static function cp_ddp_fs_opt_in()
        {
            
            if ( !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( 'You do not have sufficient permissions to perform this action.' );
                return;
            }
            
            $nonce = sanitize_text_field( $_POST['opt_nonce'] );
            $choice = sanitize_text_field( $_POST['choice'] );
            // Verify nonce.
            
            if ( empty($nonce) || !wp_verify_nonce( $nonce, 'cp-ddp-freemius-opt' ) ) {
                // Nonce verification failed.
                echo  wp_json_encode( array(
                    'success' => false,
                    'message' => esc_html__( 'Nonce verification failed.', 'delete-duplicate-posts' ),
                ) ) ;
                exit;
            }
            
            // Check if choice is not empty.
            
            if ( !empty($choice) ) {
                
                if ( 'yes' === $choice ) {
                    
                    if ( !is_multisite() ) {
                        ddp_fs()->opt_in();
                        // Opt in.
                    } else {
                        // Get sites.
                        $sites = \Freemius::get_sites();
                        $sites_data = array();
                        if ( !empty($sites) ) {
                            foreach ( $sites as $site ) {
                                $sites_data[] = ddp_fs()->get_site_info( $site );
                            }
                        }
                        ddp_fs()->opt_in(
                            false,
                            false,
                            false,
                            false,
                            false,
                            false,
                            false,
                            false,
                            $sites_data
                        );
                    }
                    
                    // Update freemius state.
                    update_site_option( CP_DDP_FREEMIUS_STATE, 'in' );
                } elseif ( 'no' === $choice ) {
                    
                    if ( !is_multisite() ) {
                        ddp_fs()->skip_connection();
                        // Opt out.
                    } else {
                        ddp_fs()->skip_connection( null, true );
                        // Opt out for all websites.
                    }
                    
                    // Update freemius state.
                    update_site_option( CP_DDP_FREEMIUS_STATE, 'skipped' );
                }
                
                echo  wp_json_encode( array(
                    'success' => true,
                    'message' => esc_html__( 'Freemius opt choice selected.', 'delete-duplicate-posts' ),
                ) ) ;
            } else {
                echo  wp_json_encode( array(
                    'success' => false,
                    'message' => esc_html__( 'Freemius opt choice not found.', 'delete-duplicate-posts' ),
                ) ) ;
            }
            
            exit;
        }
        
        /**
         * ddp_action_admin_notices.
         *
         * @author   Lars Koudal
         * @since    v0.0.1
         * @version  v1.0.0  Tuesday, January 12th, 2021.
         * @access   public static
         * @return   void
         */
        public static function ddp_action_admin_notices()
        {
            $screen = get_current_screen();
            
            if ( \PAnD::is_admin_notice_active( 'ddp-newsletter-180' ) && in_array( $screen->id, array( 'dashboard', 'tools_page_delete-duplicate-posts', 'plugins' ), true ) ) {
                $current_user = wp_get_current_user();
                ?>
				<div id="cp-ddp-newsletter" data-dismissible="ddp-newsletter-180" class="updated notice notice-success is-dismissible">
					<h3>Stay Ahead with Our Newsletter</h3>
					<p>Sign up to receive the latest tips and updates directly to your inbox. Ensure your WordPress site remains efficient and duplicate-free!</p>
					<form class="ml-block-form" action="https://assets.mailerlite.com/jsonp/16490/forms/106309157552916248/subscribe" data-code="" method="post" target="_blank">
						<input type="text" class="form-control" data-inputmask="" name="fields[name]" placeholder="Name" autocomplete="given-name" value="<?php 
                echo  esc_html( $current_user->display_name ) ;
                ?>" required="required">
						<input type="email" class="form-control" data-inputmask="" name="fields[email]" placeholder="Email" autocomplete="email" value="<?php 
                echo  esc_html( $current_user->user_email ) ;
                ?>" required="required">
						<input type="hidden" name="fields[signupsource]" value="PluginInstall">
						<input type="hidden" name="ml-submit" value="1">
						<input type="hidden" name="anticsrf" value="true">
						<button type="submit" class="button button-primary button-small">Subscribe</button>
					</form>
					</br>
				</div>
				<?php 
            }
            
            if ( 'tools_page_delete-duplicate-posts' !== $screen->id ) {
                return;
            }
            // Check anonymous mode.
            if ( 'anonymous' === get_site_option( CP_DDP_FREEMIUS_STATE, 'anonymous' ) ) {
                // If user manually opt-out then don't show the notice.
                if ( ddp_fs()->is_anonymous() && ddp_fs()->is_not_paying() && ddp_fs()->has_api_connectivity() ) {
                    if ( !is_multisite() || is_multisite() && is_network_admin() ) {
                        
                        if ( \PAnD::is_admin_notice_active( 'cp-ddp-improve-notice-30' ) ) {
                            ?>
							<div id="cp-ddp-freemius" data-dismissible="cp-ddp-improve-notice-30" class="notice notice-success is-dismissible">
								<h3><?php 
                            esc_html_e( 'Help Delete Duplicate Posts improve!', 'delete-duplicate-posts' );
                            ?></h3>

								<p>
									<?php 
                            echo  esc_html__( 'Gathering non-sensitive diagnostic data about the plugin install helps us improve the plugin.', 'delete-duplicate-posts' ) . ' <a href="' . esc_url( 'https://cleverplugins.com/docs/install/non-sensitive-diagnostic-data/' ) . '" target="_blank" rel="noopener">' . esc_html__( 'Read more about what we collect.', 'delete-duplicate-posts' ) . '</a>' ;
                            ?>
								</p>

								<p>
									<?php 
                            // translators:
                            printf( esc_html__( 'If you opt-in, some data about your usage of %1$s will be sent to Freemius.com. If you skip this, that\'s okay! %1$s will still work just fine.', 'delete-duplicate-posts' ), '<b>Delete Duplicate Posts</b>' );
                            ?>
								</p>
								<p>
									<a href="javascript:;" class="button button-primary" onclick="cp_ddp_freemius_opt_in(this)" data-opt="yes"><?php 
                            esc_html_e( 'Sure, opt-in', 'delete-duplicate-posts' );
                            ?></a>

									<a href="javascript:;" class="button dismiss-this"><?php 
                            esc_html_e( 'No, thank you', 'delete-duplicate-posts' );
                            ?></a>
								</p>
								<input type="hidden" id="cp-ddp-freemius-opt-nonce" value="<?php 
                            echo  esc_attr( wp_create_nonce( 'cp-ddp-freemius-opt' ) ) ;
                            ?>" />

							</div>
				<?php 
                        }
                    
                    }
                }
            }
            // Leave if it is not time to ask for review.
            if ( !\PAnD::is_admin_notice_active( 'ddp-leavereview-180' ) ) {
                return;
            }
            $totaldeleted = get_option( 'ddp_deleted_duplicates' );
            
            if ( false !== $totaldeleted && 0 < $totaldeleted ) {
                $totaldeleted = number_format_i18n( $totaldeleted );
                ?>
				<div id="cp-ddp-reviewlink" data-dismissible="ddp-leavereview-180" class="updated notice notice-success is-dismissible">
					<h3>
						<?php 
                // translators: Total number of deleted duplicates
                printf( esc_html__( '%s duplicates deleted!', 'delete-duplicate-posts' ), esc_html( $totaldeleted ) );
                ?>
					</h3>
					<p>
						<?php 
                // translators: Asking for a review text
                printf( esc_html__( "Hey, I noticed this plugin has deleted %s duplicate posts for you - that's awesome! Could you please do me a BIG favor and give it a 5-star rating on WordPress? Just to help us spread the word and boost our motivation.", 'delete-duplicate-posts' ), esc_html( $totaldeleted ) );
                ?>
					</p>

					<p>
						<a href="https://wordpress.org/support/plugin/delete-duplicate-posts/reviews/?filter=5#new-post" class="button-primary" target="_blank" rel="noopener">Ok, you deserve it</a>
						<span class="dashicons dashicons-calendar"></span><a href="#" class="cp-ddp-dismiss-review-notice dismiss-this" target="_blank" rel="noopener">Nope, maybe later</a>
						<span class="dashicons dashicons-smiley"></span><a href="#" class="cp-ddp-dismiss-review-notice dismiss-this" target="_blank" rel="noopener">I already did</a>
					</p>
				</div>
			<?php 
            }
        
        }
        
        /**
         * delete_duplicates_ajax.
         *
         * @author  Lars Koudal
         * @author  Unknown
         * @since   v0.0.1
         * @version v1.0.0  Tuesday, January 12th, 2021.
         * @version v1.0.1  Tuesday, October 31st, 2023.
         * @version v1.0.2  Wednesday, November 1st, 2023.
         * @access  public static
         * @param   boolean $return Default: false
         * @return  void
         */
        public static function delete_duplicates_ajax( $return_data = false )
        {
            
            if ( !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( 'You do not have sufficient permissions to perform this action.' );
                return;
            }
            
            check_ajax_referer( 'cp_ddp_delete_loglines' );
            self::log( __( 'Cleaning duplicates', 'delete-duplicate-posts' ) );
            $checked_posts = array();
            if ( isset( $_POST['checked_posts'] ) && is_array( $_POST['checked_posts'] ) ) {
                foreach ( $_POST['checked_posts'] as $cp ) {
                    $checked_posts[] = array(
                        'ID'    => intval( $cp['ID'] ),
                        'orgID' => intval( $cp['orgID'] ),
                    );
                }
            }
            
            if ( 0 < count( $checked_posts ) ) {
                self::cleandupes( true, $checked_posts );
            } else {
                // send json with error message that there were not selected rows
                wp_send_json_error( __( 'No duplicates were selected?', 'delete-duplicate-posts' ) );
            }
            
            wp_send_json_success();
        }
        
        /**
         * return_loglines_ajax.
         *
         * @author  Lars Koudal
         * @author  Unknown
         * @since   v0.0.1
         * @version v1.0.0  Tuesday, January 12th, 2021.
         * @version v1.0.1  Wednesday, November 1st, 2023.
         * @access  public static
         * @param   boolean $return Default: false
         * @return  void
         */
        public static function return_loglines_ajax( $return_data = false )
        {
            
            if ( !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( 'You do not have sufficient permissions to perform this action.' );
                return;
            }
            
            check_ajax_referer( 'cp_ddp_return_loglines' );
            global  $ddp_fs ;
            $currstep = ( isset( $_POST['step'] ) ? absint( $_POST['step'] ) : 0 );
            
            if ( !$currstep ) {
                $currstep = 0;
            } else {
                ++$currstep;
            }
            
            $json_response = array();
            if ( $currstep ) {
                $json_response['step'] = $currstep;
            }
            global  $wpdb ;
            $loglines = $wpdb->get_results( " SELECT datime, note FROM {$wpdb->prefix}ddp_log ORDER BY datime DESC LIMIT 100;" );
            
            if ( $loglines ) {
                $json_response['results'] = $loglines;
                wp_send_json_success( $json_response );
            } else {
                $json_response['msg'] = __( 'Error: Log is empty.. do something :-)', 'delete-duplicate' );
                if ( $return_data ) {
                    return $json_response;
                }
                wp_send_json_error( $json_response );
            }
            
            wp_send_json_success( $json_response );
        }
        
        /**
         * return_duplicates_ajax.
         *
         * @author	Lars Koudal
         * @author	Unknown
         * @since	v0.0.1
         * @version	v1.0.0	Friday, November 24th, 2023.
         * @access	public static
         * @return	void
         */
        public static function return_redirects_ajax__premium_only()
        {
            
            if ( !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( 'You do not have sufficient permissions to perform this action.' );
                return;
            }
            
            check_ajax_referer( 'cp_ddp_return_duplicates', true );
            global  $wpdb ;
            $redir_table = $wpdb->prefix . 'ddp_redirects';
            $response = array();
            
            if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', $redir_table ) ) === $redir_table ) {
                $totalredir_count = $wpdb->get_var( "SELECT count(id) from {$redir_table};" );
                $response['recordsTotal'] = intval( $totalredir_count );
                $start = ( isset( $_POST['start'] ) ? intval( $_POST['start'] ) : 0 );
                $length = ( isset( $_POST['length'] ) ? intval( $_POST['length'] ) : 10 );
                $searchValue = ( isset( $_POST['search']['value'] ) ? sanitize_text_field( $_POST['search']['value'] ) : '' );
                $whereClause = '';
                
                if ( !empty($searchValue) ) {
                    $whereClause = $wpdb->prepare( " WHERE inurl LIKE %s OR targeturl LIKE %s", '%' . $wpdb->esc_like( $searchValue ) . '%', '%' . $wpdb->esc_like( $searchValue ) . '%' );
                    $totalredir_count_filtered = $wpdb->get_var( "SELECT count(id) from {$redir_table}" . $whereClause );
                } else {
                    $totalredir_count_filtered = $totalredir_count;
                }
                
                $response['recordsFiltered'] = intval( $totalredir_count_filtered );
                $latest_redirs = $wpdb->get_results( $wpdb->prepare( "SELECT id, inurl, targeturl from {$redir_table} {$whereClause} ORDER BY id DESC LIMIT %d, %d;", $start, $length ), ARRAY_A );
                
                if ( $latest_redirs ) {
                    $data = array();
                    foreach ( $latest_redirs as $red ) {
                        $row = array();
                        $row['ID'] = intval( $red['id'] );
                        $row['from_url'] = esc_url( $red['inurl'] );
                        $row['target_url'] = esc_url( $red['targeturl'] );
                        $data[] = $row;
                    }
                    $response['data'] = $data;
                } else {
                    $response['data'] = array();
                }
                
                $response['draw'] = ( isset( $_POST['draw'] ) ? intval( $_POST['draw'] ) : 0 );
                wp_send_json( $response );
                die;
            } else {
                wp_send_json_error( 'Table does not exist' );
            }
        
        }
        
        /**
         * return_duplicates_ajax.
         *
         * @author  Lars Koudal
         * @author  Unknown
         * @since   v0.0.1
         * @version v1.0.0  Tuesday, January 12th, 2021.
         * @version v1.0.1  Wednesday, November 1st, 2023.
         * @access  public static
         * @return  void
         */
        public static function return_duplicates_ajax()
        {
            
            if ( !current_user_can( 'manage_options' ) ) {
                wp_send_json_error( 'You do not have sufficient permissions to perform this action.' );
                return;
            }
            
            check_ajax_referer( 'cp_ddp_return_duplicates', true );
            // Get duplicates
            $duplicates = self::return_duplicates( true );
            // Initialize DataTables response array
            $response = array(
                'draw'            => intval( $_POST['draw'] ),
                'recordsTotal'    => 0,
                'recordsFiltered' => 0,
                'data'            => array(),
            );
            
            if ( !empty($duplicates) && isset( $duplicates['dupes'] ) ) {
                $response['recordsTotal'] = $duplicates['dupescount'];
                $response['recordsFiltered'] = $duplicates['dupescount'];
                foreach ( $duplicates['dupes'] as $dupe ) {
                    if ( '' === $dupe['title'] ) {
                        $dupe['title'] = '-empty-';
                    }
                    if ( '' === $dupe['orgtitle'] ) {
                        $dupe['orgtitle'] = '-empty-';
                    }
                    $dupedetails = array(
                        'ID'           => $dupe['ID'],
                        'permalink'    => get_permalink( $dupe['ID'] ),
                        'title'        => $dupe['title'],
                        'type'         => $dupe['type'],
                        'orgID'        => $dupe['orgID'],
                        'orgtitle'     => $dupe['orgtitle'],
                        'orgpermalink' => get_permalink( $dupe['orgID'] ),
                        'status'       => $dupe['status'],
                        'why'          => $dupe['why'],
                    );
                    $response['data'][] = array(
                        'ID'        => $dupe['ID'],
                        'orgID'     => $dupe['orgID'],
                        'duplicate' => '<a href="' . esc_url( $dupedetails['permalink'] ) . '" target="_blank">' . $dupedetails['title'] . '</a> (ID #' . $dupedetails['ID'] . ') <br><small>' . $dupedetails['why'] . ' Type:' . $dupedetails['type'] . ' Status:' . $dupedetails['status'] . ')</small>',
                        'original'  => '<a href="' . $dupedetails['orgpermalink'] . '" target="_blank">' . $dupedetails['orgtitle'] . '</a> (ID #' . $dupedetails['orgID'] . ')',
                    );
                }
            }
            
            wp_send_json( $response );
            die;
        }
        
        /**
         * Converts a number to relevant unit size
         *
         * @author   Lars Koudal
         * @since    v0.0.1
         * @version  v1.0.0  Thursday, June 24th, 2021.
         * @param    mixed   $size
         * @return   void
         */
        public static function pretty_value( $size )
        {
            $unit = array(
                'b',
                'kb',
                'mb',
                'gb',
                'tb',
                'pb'
            );
            $log = log( $size, 1024 );
            $i = floor( $log );
            $num = $size / pow( 1024, $i );
            $calc = round( $num, 2 ) . ' ' . $unit[$i];
            return $calc;
        }
        
        /**
         * Returns duplicates based on current settings - internal, not used via AJAX
         *
         * @author  Lars Koudal
         * @author  Unknown
         * @since   v0.0.1
         * @version v1.0.0  Tuesday, January 12th, 2021.
         * @version v1.0.1  Wednesday, November 1st, 2023.
         * @access  public static
         * @param   boolean $return Default: false
         * @return  void
         */
        public static function return_duplicates( $return = false )
        {
            self::timerstart( 'return_duplicates' );
            $options = self::get_options();
            $comparemethod = 'titlecompare';
            $return_duplicates_time = false;
            global  $ddp_fs ;
            if ( $ddp_fs->is__premium_only() ) {
                if ( $ddp_fs->can_use_premium_code() ) {
                    
                    if ( isset( $options['ddp_method'] ) ) {
                        $comparemethod = $options['ddp_method'];
                    } else {
                        $comparemethod = 'titlecompare';
                    }
                
                }
            }
            
            if ( isset( $currstep ) ) {
                ++$currstep;
            } else {
                $currstep = 0;
            }
            
            $json_response = array();
            if ( isset( $currstep ) ) {
                $json_response['step'] = $currstep;
            }
            // @ check compare method - maybe change lookup routine?
            global  $wpdb ;
            $table_name = $wpdb->prefix . 'posts';
            $resultslimit = $options['ddp_resultslimit'];
            $viewlimit = intval( $resultslimit );
            if ( 0 === $viewlimit ) {
                $viewlimit = 9999;
            }
            $ddp_pts_arr = $options['ddp_pts'];
            
            if ( isset( $ddp_pts_arr ) && is_array( $ddp_pts_arr ) ) {
                $ddp_pts = '"' . implode( '","', $ddp_pts_arr ) . '"';
            } else {
                $ddp_pts = '';
            }
            
            $ddp_pts = rtrim( $ddp_pts, ',' );
            $post_stati = '"publish"';
            if ( $ddp_fs->is__premium_only() ) {
                
                if ( $ddp_fs->can_use_premium_code() ) {
                    $ddp_pstati = $options['ddp_pstati'];
                    
                    if ( isset( $ddp_pstati ) && is_array( $ddp_pstati ) ) {
                        $post_stati = '';
                        foreach ( $ddp_pstati as $key => $status ) {
                            $post_stati .= '"' . $status . '",';
                        }
                    } else {
                        $post_stati = '';
                    }
                    
                    $post_stati = rtrim( $post_stati, ',' );
                }
            
            }
            $order = $options['ddp_keep'];
            // verify default value has been set
            
            if ( 'oldest' !== $order ) {
                // two choices, if its not the first its the second...
                $options['ddp_keep'] = 'latest';
                $order = 'latest';
            }
            
            if ( 'oldest' === $order ) {
                $minmax = 'MIN(id)';
            }
            if ( 'latest' === $order ) {
                $minmax = 'MAX(id)';
            }
            $ddpstatuscnt = array();
            $dupescount = 0;
            
            if ( '' !== $ddp_pts ) {
                $thisquery = false;
                // **** Compare by title ****
                
                if ( 'titlecompare' === $comparemethod ) {
                    $limit = ( isset( $_POST['length'] ) ? intval( $_POST['length'] ) : 10 );
                    // Default to 10 if not set
                    $offset = ( isset( $_POST['start'] ) ? intval( $_POST['start'] ) : 0 );
                    // Default to 0 if not set
                    
                    if ( !wp_doing_ajax() && $return ) {
                        $limit = $options['ddp_resultslimit'];
                        // for returning results for cron job.
                    }
                    
                    $resultsoutput = ' LIMIT ' . intval( $limit ) . ' OFFSET ' . intval( $offset );
                    $thisquery = "SELECT * FROM (\n\t\t\t\t\t\tSELECT t1.ID, t1.post_title, t1.post_type, t1.post_status, save_this_post_id \n\t\t\t\t\t\tFROM {$table_name} AS t1 \n\t\t\t\t\t\tINNER JOIN ( \n\t\t\t\t\t\t\t\tSELECT post_title, {$minmax} AS save_this_post_id \n\t\t\t\t\t\t\t\tFROM {$table_name} \n\t\t\t\t\t\t\t\tWHERE post_type IN ( {$ddp_pts} ) \n\t\t\t\t\t\t\t\tAND post_status = 'publish' \n\t\t\t\t\t\t\t\tGROUP BY post_title \n\t\t\t\t\t\t\t\tHAVING COUNT(*) > 1 \n\t\t\t\t\t\t) AS t2 ON t1.post_title = t2.post_title \n\t\t\t\t\t\tWHERE t1.post_status = 'publish'\n\t\t\t\t\t\tORDER BY t1.post_title, t1.post_date DESC\n\t\t\t\t) AS derived_table\n\t\t\t\tWHERE ID != save_this_post_id\n\t\t\t\t{$resultsoutput}";
                    $json_response['lookup_query'] = $thisquery;
                    $dupes = $wpdb->get_results( $thisquery, ARRAY_A );
                    // here we get total dupes - not cute, but the other approach not working.
                    $total_dupes = $wpdb->get_var( "SELECT COUNT(*) FROM (\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tSELECT t1.ID, t1.post_title, t1.post_type, t1.post_status, save_this_post_id \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tFROM {$table_name} AS t1 \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tINNER JOIN ( \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tSELECT post_title, {$minmax} AS save_this_post_id \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tFROM {$table_name} \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tWHERE post_type IN (  {$ddp_pts} ) \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND post_type NOT IN ('nav_menu_item') \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND post_status IN ( {$post_stati} ) \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tGROUP BY post_title \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tHAVING COUNT(*)>1 \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t) AS t2 \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tON t1.post_title = t2.post_title \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND post_status IN ( {$post_stati} )\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t) AS derived_table\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tWHERE ID != save_this_post_id" );
                    $json_response['lookup_error'] = htmlspecialchars( $wpdb->last_error, ENT_QUOTES );
                    
                    if ( '' !== $wpdb->last_error ) {
                        $last_error = htmlspecialchars( $wpdb->last_error, ENT_QUOTES );
                        self::log( 'Look up error: ' . $last_error );
                    }
                    
                    
                    if ( $dupes ) {
                        $json_response['dupescount'] = $total_dupes;
                        $stepcount = 0;
                        foreach ( $dupes as $dupe ) {
                            $mystatus = $dupe['post_status'];
                            
                            if ( isset( $ddpstatuscnt[$mystatus] ) ) {
                                $ddpstatuscnt[$mystatus] = $ddpstatuscnt[$mystatus] + 1;
                            } else {
                                $ddpstatuscnt[$mystatus] = 1;
                            }
                            
                            // Only save the dupes
                            
                            if ( $dupe['ID'] !== $dupe['save_this_post_id'] ) {
                                $dupedetails = array(
                                    'ID'           => $dupe['ID'],
                                    'permalink'    => get_permalink( $dupe['ID'] ),
                                    'title'        => $dupe['post_title'],
                                    'type'         => $dupe['post_type'],
                                    'orgID'        => $dupe['save_this_post_id'],
                                    'orgtitle'     => $dupe['post_title'],
                                    'orgpermalink' => get_permalink( $dupe['save_this_post_id'] ),
                                    'status'       => $dupe['post_status'],
                                    'why'          => 'Post ID ' . $dupe['ID'] . ' has the same title as Post ID ' . $dupe['save_this_post_id'],
                                );
                                $json_response['dupes'][] = $dupedetails;
                            }
                            
                            ++$stepcount;
                        }
                    }
                
                }
                
                if ( $ddp_fs->is__premium_only() ) {
                    if ( $ddp_fs->can_use_premium_code() ) {
                        // ********************* COMPARE BY META TAG ****************
                        // ddp_compare_metatag
                        if ( 'metacompare' === $comparemethod ) {
                            
                            if ( !$options['ddp_compare_metatag'] ) {
                                wp_send_json_error( __( 'Error, choose which meta_key to look for duplicate values in.', 'delete-duplicate-posts' ) );
                            } else {
                                $limit = ( isset( $_POST['length'] ) ? intval( $_POST['length'] ) : 10 );
                                // Default to 10 if not set
                                $offset = ( isset( $_POST['start'] ) ? intval( $_POST['start'] ) : 0 );
                                // Default to 0 if not set
                                if ( 'oldest' === $order ) {
                                    $minmax = 'MIN(p.ID)';
                                }
                                if ( 'latest' === $order ) {
                                    $minmax = 'MAX(p.ID)';
                                }
                                $post_stati = array( 'publish' );
                                // Default value
                                if ( $ddp_fs->is__premium_only() ) {
                                    
                                    if ( $ddp_fs->can_use_premium_code() ) {
                                        $ddp_pstati = $options['ddp_pstati'];
                                        if ( isset( $ddp_pstati ) && is_array( $ddp_pstati ) ) {
                                            $post_stati = $ddp_pstati;
                                        }
                                    }
                                
                                }
                                // Prepare the ddp_pts for use in SQL IN clause
                                $ddp_pts_arr = $options['ddp_pts'];
                                
                                if ( isset( $ddp_pts_arr ) && is_array( $ddp_pts_arr ) ) {
                                    $ddp_pts = $ddp_pts_arr;
                                } else {
                                    $ddp_pts = array();
                                }
                                
                                // Escape and prepare the post_stati and ddp_pts for direct insertion into the SQL query
                                $escaped_post_stati = implode( ', ', array_map( function ( $status ) {
                                    return "'" . esc_sql( $status ) . "'";
                                }, $post_stati ) );
                                $escaped_ddp_pts = implode( ', ', array_map( function ( $type ) {
                                    return "'" . esc_sql( $type ) . "'";
                                }, $ddp_pts ) );
                                $query = $wpdb->prepare(
                                    "SELECT p.ID as ID, original.ID as orgID, p.post_type, p.post_status, pm.meta_value as mvalue\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tFROM {$wpdb->prefix}posts as p\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tLEFT JOIN {$wpdb->prefix}postmeta pm ON ( p.ID = pm.post_id )\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tINNER JOIN (\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tSELECT MIN(p.ID) as ID, pm.meta_value as mvalue\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tFROM {$wpdb->prefix}posts as p\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tLEFT JOIN {$wpdb->prefix}postmeta pm ON ( p.ID = pm.post_id )\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tWHERE p.post_status IN ({$escaped_post_stati})\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND p.post_type NOT IN ('nav_menu_item')\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND pm.meta_key = %s\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tGROUP BY pm.meta_value\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tHAVING COUNT(pm.meta_value) > 1\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t) AS original ON original.mvalue = pm.meta_value AND original.ID != p.ID\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tWHERE p.post_status IN ({$escaped_post_stati})\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND p.post_type NOT IN ('nav_menu_item')\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND p.post_type IN ({$escaped_ddp_pts})\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND pm.meta_key = %s\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tLIMIT %d OFFSET %d;",
                                    $options['ddp_compare_metatag'],
                                    $options['ddp_compare_metatag'],
                                    $limit,
                                    $offset
                                );
                                // Here we get the total amount of dupes - Wish we could get it in one query.
                                $total_dupes_query = $wpdb->prepare( "SELECT COUNT(*) FROM (\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tSELECT p.ID\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tFROM {$wpdb->prefix}posts as p\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tINNER JOIN {$wpdb->prefix}postmeta pm ON p.ID = pm.post_id\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tINNER JOIN (\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tSELECT MIN(p.ID) as ID, pm.meta_value\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tFROM {$wpdb->prefix}posts as p\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tINNER JOIN {$wpdb->prefix}postmeta pm ON p.ID = pm.post_id\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tWHERE p.post_status IN ({$escaped_post_stati})\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND p.post_type NOT IN ('nav_menu_item')\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND pm.meta_key = %s\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tGROUP BY pm.meta_value\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tHAVING COUNT(pm.meta_value) > 1\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t) AS original ON original.meta_value = pm.meta_value AND original.ID != p.ID\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tWHERE p.post_status IN ({$escaped_post_stati})\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND p.post_type NOT IN ('nav_menu_item')\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND p.post_type IN ({$escaped_ddp_pts})\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tAND pm.meta_key = %s\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t) AS subquery", $options['ddp_compare_metatag'], $options['ddp_compare_metatag'] );
                                $total_dupes = $wpdb->get_var( $total_dupes_query );
                                $json_response['lookup_query'] = $query;
                                if ( $options['ddp_debug'] ) {
                                    self::log( 'DEBUG: SQL ' . esc_attr( $query ) );
                                }
                                $dupelist = $wpdb->get_results( $query );
                                
                                if ( '' !== $wpdb->last_error ) {
                                    $last_error = htmlspecialchars( $wpdb->last_error, ENT_QUOTES );
                                    self::log( 'Look up error: ' . esc_html( $last_error ) );
                                }
                                
                                
                                if ( $dupelist ) {
                                    $dupe_arr = array();
                                    foreach ( $dupelist as $dupe ) {
                                        $escaped_val = md5( $dupe->mvalue );
                                        
                                        if ( !isset( $dupe_arr[$escaped_val] ) ) {
                                            $dupe_arr[$escaped_val]['count'] = 1;
                                            $dupe_arr[$escaped_val]['val'] = $dupe->mvalue;
                                        } else {
                                            $dupe_arr[$escaped_val]['count'] = $dupe_arr[$escaped_val]['count'] + 1;
                                        }
                                        
                                        $dupe_arr[$escaped_val]['ids'][$dupe->ID] = array(
                                            'type'   => $dupe->post_type,
                                            'status' => $dupe->post_status,
                                            'ID'     => $dupe->ID,
                                            'orgID'  => $dupe->orgID,
                                            'value'  => $dupe->mvalue,
                                        );
                                    }
                                    // find min or max and then populate the duplicate array
                                    foreach ( $dupe_arr as $fa ) {
                                        $idlist = $fa['ids'];
                                        ksort( $idlist );
                                        
                                        if ( $idlist ) {
                                            
                                            if ( 'latest' === $order ) {
                                                $lastelem = end( $idlist );
                                                $ignore_id = $lastelem['ID'];
                                            }
                                            
                                            
                                            if ( 'oldest' === $order ) {
                                                $lastelem = reset( $idlist );
                                                $ignore_id = $lastelem['ID'];
                                            }
                                            
                                            foreach ( $idlist as $key => $so ) {
                                                
                                                if ( intval( $ignore_id ) !== intval( $key ) ) {
                                                    $dupedetails = array(
                                                        'ID'           => $so['ID'],
                                                        'permalink'    => get_permalink( $so['ID'] ),
                                                        'title'        => get_the_title( $so['ID'] ),
                                                        'status'       => get_post_status( $so['ID'] ),
                                                        'type'         => $so['type'],
                                                        'orgID'        => $ignore_id,
                                                        'orgtitle'     => get_the_title( $ignore_id ),
                                                        'orgpermalink' => get_permalink( $ignore_id ),
                                                        'why'          => ' meta_key <code>' . esc_attr( $options['ddp_compare_metatag'] ) . '</code> has the same meta_value <code>' . esc_attr( $fa['val'] ) . '</code>',
                                                    );
                                                    $mystatus = $dupedetails['status'];
                                                    
                                                    if ( isset( $ddpstatuscnt[$mystatus] ) ) {
                                                        $ddpstatuscnt[$mystatus] = $ddpstatuscnt[$mystatus] + 1;
                                                    } else {
                                                        $ddpstatuscnt[$mystatus] = 1;
                                                    }
                                                    
                                                    $json_response['dupes'][] = $dupedetails;
                                                    $json_response['dupescount'] = $total_dupes;
                                                }
                                            
                                            }
                                        }
                                    
                                    }
                                }
                            
                            }
                        
                        }
                    }
                }
                $statusdata = '';
                
                if ( is_array( $ddpstatuscnt ) && count( $ddpstatuscnt ) > 1 ) {
                    $statusdata .= '(';
                    foreach ( $ddpstatuscnt as $key => $dsc ) {
                        $statusdata .= $key . ': ' . number_format_i18n( $dsc ) . ', ';
                    }
                    $statusdata = rtrim( $statusdata, ', ' );
                    $statusdata .= ')';
                }
                
                $return_duplicates_time = self::timerstop( 'return_duplicates' );
                
                if ( $options['ddp_debug'] ) {
                    $max = 5;
                    
                    if ( isset( $json_response['dupes'] ) ) {
                        $idlist = array();
                        $step = 0;
                        foreach ( $json_response['dupes'] as $dupe ) {
                            
                            if ( $step <= $max ) {
                                $details = '';
                                if ( isset( $dupe['ID'] ) ) {
                                    $details .= 'ID: ' . $dupe['ID'] . ' ';
                                }
                                if ( isset( $dupe['title'] ) ) {
                                    $details .= ' title: "' . $dupe['title'] . '" ';
                                }
                                if ( isset( $dupe['permalink'] ) ) {
                                    $details .= 'Permalink: ' . $dupe['permalink'] . ' ';
                                }
                                if ( isset( $dupe['status'] ) ) {
                                    $details .= 'Status: ' . $dupe['status'] . ' ';
                                }
                                if ( isset( $dupe['type'] ) ) {
                                    $details .= 'Type: ' . $dupe['type'] . ' ';
                                }
                                if ( isset( $dupe['orgID'] ) ) {
                                    $details .= 'orgID: ' . $dupe['orgID'] . ' ';
                                }
                                if ( isset( $dupe['orgtitle'] ) ) {
                                    $details .= 'orgtitle: ' . $dupe['orgtitle'] . ' ';
                                }
                                if ( isset( $dupe['orgpermalink'] ) ) {
                                    $details .= ' orgpermalink: ' . $dupe['orgpermalink'] . ' ';
                                }
                                self::log( $details );
                            }
                            
                            ++$step;
                        }
                    }
                
                }
                
                self::log( $json_response['dupescount'] . ' duplicates found in ' . $return_duplicates_time . ' sec. ' . $statusdata . ' Mem usage: ' . self::pretty_value( memory_get_peak_usage( true ) ) );
            } else {
                $json_response['msg'] = __( 'Error: Choose post types to check.', 'delete-duplicate-posts' );
                $return_duplicates_time = self::timerstop( 'return_duplicates' );
                $json_response['time'] = $return_duplicates_time . ' sec';
                if ( $return ) {
                    return $json_response;
                }
                wp_send_json_error( $json_response );
            }
            
            if ( !$return_duplicates_time ) {
                $return_duplicates_time = self::timerstop( 'return_duplicates' );
            }
            $json_response['msg'] = number_format_i18n( $json_response['dupescount'] ) . ' duplicates found. Time: ' . esc_html( $return_duplicates_time ) . ' sec.';
            // @todo i8n
            if ( $return ) {
                return $json_response;
            }
            wp_send_json_success( $json_response );
        }
        
        /**
         * create_redirect.
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Friday, July 2nd, 2021.
         * @version v1.0.1  Tuesday, October 18th, 2022.
         * @access  public static
         * @param   mixed   $inurl
         * @param   mixed   $targeturl
         * @param   integer $code       Default: 301
         * @return  void
         */
        public static function create_redirect( $inurl, $targeturl, $code = 301 )
        {
            global  $wpdb, $ddp_fs ;
            if ( $ddp_fs->is__premium_only() ) {
                
                if ( $ddp_fs->can_use_premium_code() ) {
                    $table_name = $wpdb->prefix . 'ddp_redirects';
                    
                    if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', $table_name ) ) !== $table_name ) {
                        $sql = "CREATE TABLE {$table_name} (id bigint(20) NOT NULL AUTO_INCREMENT,datime timestamp NOT NULL DEFAULT '0000-00-00 00:00:00' ON UPDATE CURRENT_TIMESTAMP, inurl varchar(1024) NOT NULL, targeturl varchar(1024) NOT NULL, httpcode varchar(3), PRIMARY KEY (id));";
                        require_once ABSPATH . 'wp-admin/includes/upgrade.php';
                        dbDelta( $sql );
                    }
                    
                    $wpdb->insert( $table_name, array(
                        'datime'    => current_time( 'Y-m-d H:i:s' ),
                        'inurl'     => wp_make_link_relative( $inurl ),
                        'targeturl' => $targeturl,
                        'httpcode'  => $code,
                    ) );
                    // translators: Notice for creating a redirect from a to b. Value 1 is HTTP status code (ie 301), 2 is from and 3 is target URL.
                    self::log( sprintf(
                        __( 'Created %1$s redirect from %2$s to %3$s', 'delete-duplicate-posts' ),
                        $code,
                        $inurl,
                        $targeturl
                    ) );
                }
            
            }
        }
        
        /**
         * Return default options
         *
         * @author   Lars Koudal
         * @since    v0.0.1
         * @version  v1.0.0  Friday, July 2nd, 2021.
         * @access   public static
         * @return   mixed
         */
        public static function default_options()
        {
            $defaults = array(
                'ddp_running'              => 'false',
                'ddp_keep'                 => 'oldest',
                'ddp_limit'                => 50,
                'ddp_pts'                  => array( 'post', 'page' ),
                'ddp_statusmail_recipient' => '',
                'ddp_statusmail'           => 0,
                'ddp_resultslimit'         => 0,
                'ddp_enabled'              => 0,
                'ddp_pstati'               => array( 'publish' ),
                'ddp_debug'                => 0,
                'ddp_redirects'            => 0,
            );
            return $defaults;
        }
        
        /**
         * get plugin's options
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @return  mixed
         */
        public static function get_options()
        {
            $options = get_option( self::$options_name, array() );
            if ( !is_array( $options ) ) {
                $options = array();
            }
            $options = array_merge( self::default_options(), $options );
            return $options;
        }
        
        /**
         * add_freemius_extra_permission.
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   mixed   $permissions
         * @return  mixed
         */
        public static function add_freemius_extra_permission( $permissions )
        {
            $permissions['helpscout'] = array(
                'icon-class' => 'dashicons dashicons-sos',
                'label'      => 'Help Scout',
                'desc'       => __( 'Rendering Help Scouts beacon for easy help and support', 'delete-duplicate-posts' ),
                'priority'   => 16,
            );
            $permissions['newsletter'] = array(
                'icon-class' => 'dashicons dashicons-email-alt2',
                'label'      => 'Newsletter',
                'desc'       => __( 'Your email is added to cleverplugins.com newsletter. Unsubscribe any time.', 'delete-duplicate-posts' ),
                'priority'   => 18,
            );
            return $permissions;
        }
        
        /**
         * Fetch plugin version from plugin PHP header
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @return  mixed
         */
        public static function get_plugin_version()
        {
            $plugin_data = get_file_data( __FILE__, array(
                'version' => 'Version',
            ), 'plugin' );
            return $plugin_data['version'];
        }
        
        /**
         * timerstart.
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   mixed   $watchname
         * @return  void
         */
        public static function timerstart( $watchname )
        {
            set_transient( 'ddp_' . $watchname, microtime( true ), 60 * 60 * 1 );
        }
        
        /**
         * timerstop.
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   mixed   $watchname
         * @param   integer $digits     Default: 3
         * @return  mixed
         */
        public static function timerstop( $watchname, $digits = 3 )
        {
            $return = round( microtime( true ) - get_transient( 'ddp_' . $watchname ), $digits );
            delete_transient( 'ddp_' . $watchname );
            return $return;
        }
        
        // cron version of the cleaning dupes function
        public static function cron_cleandupes()
        {
        }
        
        /**
         * Clean duplicates - not AJAX version
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   boolean $manualrun  Default: false
         * @param   mixed   $to_delete  Default: array()
         * @return  void
         */
        public static function cleandupes( $manualrun = false, $to_delete = array() )
        {
            global  $wpdb, $ddp_fs ;
            self::timerstart( 'ddp_totaltime' );
            // start total timer
            $options = self::get_options();
            $options['ddp_running'] = true;
            self::save_options( $options );
            
            if ( !$manualrun ) {
                self::log( __( 'Automatic CRON job running.', 'delete-duplicate-posts' ) );
            } else {
                self::log( __( 'Manually cleaning.', 'delete-duplicate-posts' ) );
            }
            
            // what to do with a manual run - no notices
            
            if ( count( $to_delete ) > 0 ) {
                $lookup_arr = array();
                foreach ( $to_delete as $td ) {
                    $new_item = array();
                    $new_item['ID'] = $td['ID'];
                    $new_item['orgID'] = $td['orgID'];
                    $new_item['type'] = get_post_type( $td['ID'] );
                    $new_item['title'] = get_the_title( $td['ID'] );
                    $lookup_arr['dupes'][] = $new_item;
                }
                $dupes = $lookup_arr;
            } else {
                $dupes = self::return_duplicates( true );
            }
            
            $resultnote = '';
            $dispcount = 0;
            if ( isset( $dupes['dupes'] ) ) {
                foreach ( $dupes['dupes'] as $dupe ) {
                    $postid = $dupe['ID'];
                    $title = substr( $dupe['title'], 0, 35 );
                    
                    if ( $postid ) {
                        self::timerstart( 'deletepost_' . $postid );
                        if ( $ddp_fs->is__premium_only() ) {
                            if ( $ddp_fs->can_use_premium_code() ) {
                                
                                if ( $options['ddp_redirects'] ) {
                                    $perma = get_permalink( $dupe['ID'] );
                                    $orgperma = get_permalink( $dupe['orgID'] );
                                    self::create_redirect( $perma, $orgperma );
                                }
                            
                            }
                        }
                        /* @todo - implement a premium option to permanently delete or just use the WP setting.
                        			
                        			Options: Use WP setting (default), Delete Permantently, Trash posts (if enabled in WP)
                        			
                        			*/
                        $deleteresult = wp_trash_post( $postid );
                        $timespent = self::timerstop( 'deletepost_' . $postid );
                        ++$dispcount;
                        $totaldeleted = get_option( 'ddp_deleted_duplicates' );
                        
                        if ( false !== $totaldeleted ) {
                            ++$totaldeleted;
                            update_option( 'ddp_deleted_duplicates', $totaldeleted );
                        } else {
                            update_option( 'ddp_deleted_duplicates', 1 );
                        }
                        
                        if ( $options['ddp_debug'] ) {
                            // translators: Debug notice. 1: type of duplicate. 2: The title of the post. 3: The ID. 4: Time spent deleting.
                            self::log( sprintf(
                                __( 'DEBUG: Deleted %1$s %2$s (id: %3$s) in %4$s sec.', 'delete-duplicate-posts' ),
                                $dupe['type'],
                                $title,
                                $postid,
                                $timespent
                            ) );
                        }
                    }
                
                }
            }
            $totaltimespent = self::timerstop( 'ddp_totaltime' );
            self::log( sprintf( __( 'A total of %1$s duplicate posts were deleted in %2$s sec.', 'delete-duplicate-posts' ), $dispcount, $totaltimespent ) );
            // Mail logic...
            
            if ( 0 < $dispcount && $options['ddp_statusmail'] ) {
                $blogurl = site_url();
                $recipient = $options['ddp_statusmail_recipient'];
                // translators:
                $messagebody = sprintf( __( 'Hi Admin, I have deleted <strong>%1$d</strong> duplicated posts on your blog, %2$s.', 'delete-duplicate-posts' ), $dispcount, $blogurl );
                $messagebody .= '<br><br>' . __( 'You are receiving this e-mail because you have turned on e-mail notifications by the plugin' ) . ' <a href="https://cleverplugins.com/delete-duplicate-posts/" target="_blank" rel="noopener">Delete Duplicate Posts</a>';
                $messagebody .= "<br><br>Made by <a href='https://cleverplugins.com' target='_blank' rel='noopener'>cleverplugins.com</a>";
                $mailstatus = false;
                
                if ( is_email( $recipient ) ) {
                    $mailstatus = wp_mail( $recipient, __( 'Deleted Duplicate Posts Status', 'delete-duplicate-posts' ), $messagebody );
                    if ( $options['ddp_debug'] ) {
                        // translators: Debug message with email recipient.
                        self::log( sprintf( __( 'DEBUG: Sending email to : %s ', 'delete-duplicate-posts' ), $recipient ) );
                    }
                    if ( $mailstatus ) {
                        // translators:
                        self::log( sprintf( __( 'Status email sent to %s.', 'delete-duplicate-posts' ), $recipient ) );
                    }
                } else {
                    // translators:
                    self::log( sprintf( __( 'Not a vaild email %s.', 'delete-duplicate-posts' ), $recipient ) );
                }
            
            }
            
            $options['ddp_running'] = false;
            self::save_options( $options );
            // Lets return a response
            
            if ( 0 === $manualrun && !wp_doing_ajax() ) {
                $json_response = array(
                    'msg' => sprintf( esc_html__( 'A total of %s duplicates were deleted.', 'delete-duplicate-posts' ), intval( $dispcount ) ),
                );
                wp_send_json_success( $json_response );
            }
        
        }
        
        /**
         * add_cron_intervals.
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   mixed   $schedules
         * @return  mixed
         */
        public static function add_cron_intervals( $schedules )
        {
            $schedules['5min'] = array(
                'interval' => 300,
                'display'  => __( 'Every 5 minutes', 'delete-duplicate-posts' ),
            );
            $schedules['10min'] = array(
                'interval' => 600,
                'display'  => __( 'Every 10 minutes', 'delete-duplicate-posts' ),
            );
            $schedules['15min'] = array(
                'interval' => 900,
                'display'  => __( 'Every 15 minutes', 'delete-duplicate-posts' ),
            );
            $schedules['30min'] = array(
                'interval' => 1800,
                'display'  => __( 'Every 30 minutes', 'delete-duplicate-posts' ),
            );
            return $schedules;
        }
        
        /**
         * Log a notification to the database
         *
         * @author   Lars Koudal
         * @since    v0.0.1
         * @version  v1.0.0  Monday, January 11th, 2021.
         * @access   public static
         * @param    mixed   $text
         * @return   void
         */
        public static function log( $text )
        {
            global  $wpdb ;
            $ddp_logtable = $wpdb->prefix . 'ddp_log';
            $wpdb->insert( $ddp_logtable, array(
                'datime' => current_time( 'mysql' ),
                'note'   => $text,
            ), array( '%s', '%s' ) );
            // When over 1000 entries, strip down to 500.
            $total = (int) $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->prefix}ddp_log;" );
            // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
            
            if ( $total > 1 ) {
                $targettime = $wpdb->get_var( "SELECT `datime` from {$wpdb->prefix}ddp_log order by `datime` DESC limit 500,1;" );
                // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
                $wpdb->query( $wpdb->prepare( "DELETE from {$wpdb->prefix}ddp_log  where `datime` < %s", $targettime ) );
            }
        
        }
        
        /**
         * Enqueues scripts and styles
         *
         * @author   Lars Koudal
         * @since    v0.0.1
         * @version  v1.0.0  Monday, January 11th, 2021.
         * @access   public static
         * @return   void
         */
        public static function admin_enqueue_scripts()
        {
            $screen = get_current_screen();
            
            if ( is_object( $screen ) && 'tools_page_delete-duplicate-posts' === $screen->id ) {
                $pluginver = self::get_plugin_version();
                wp_enqueue_script( 'jquery' );
                wp_enqueue_script( 'jquery-ui-tabs' );
                wp_enqueue_style(
                    'delete-duplicate-posts',
                    plugins_url( '/css/delete-duplicate-posts-min.css', __FILE__ ),
                    array(),
                    $pluginver
                );
                wp_enqueue_script(
                    'dataTables',
                    plugins_url( '/js/DataTables/datatables.min.js', __FILE__ ),
                    array(),
                    $pluginver
                );
                wp_enqueue_style(
                    'dataTables',
                    plugins_url( '/js/DataTables/datatables.min.css', __FILE__ ),
                    array(),
                    $pluginver
                );
                wp_register_script(
                    'delete-duplicate-posts',
                    plugins_url( '/js/delete-duplicate-posts.js', __FILE__ ),
                    array( 'jquery', 'dataTables' ),
                    $pluginver,
                    true
                );
                $js_vars = array(
                    'nonce'                => wp_create_nonce( 'cp_ddp_return_duplicates' ),
                    'loglines_nonce'       => wp_create_nonce( 'cp_ddp_return_loglines' ),
                    'deletedupes_nonce'    => wp_create_nonce( 'cp_ddp_delete_loglines' ),
                    'text_areyousure'      => __( 'Are you sure you want to delete duplicates? There is no undo feature.', 'delete-duplicate-posts' ),
                    'text_selectsomething' => __( 'You have to select which duplicates to delete. Tip: You can click the top or bottom checkbox to select all.', 'delete-duplicate-posts' ),
                );
                wp_localize_script( 'delete-duplicate-posts', 'cp_ddp', $js_vars );
                wp_enqueue_script( 'delete-duplicate-posts' );
            }
        
        }
        
        /**
         * Create plugin tables
         *
         * @author	Lars Koudal
         * @author	Unknown
         * @since	v0.0.1
         * @version	v1.0.0	Monday, January 11th, 2021.	
         * @version	v1.0.1	Sunday, July 17th, 2022.	
         * @version	v1.0.2	Sunday, December 3rd, 2023.
         * @access	public static
         * @return	void
         */
        public static function create_table()
        {
            global  $wpdb, $ddp_fs ;
            require_once ABSPATH . 'wp-admin/includes/upgrade.php';
            $log_table_name = $wpdb->prefix . 'ddp_log';
            $sql_log = "CREATE TABLE {$log_table_name} (\n\t\t\t\t\tid bigint(20) NOT NULL AUTO_INCREMENT,\n\t\t\t\t\tdatime timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,\n\t\t\t\t\tnote tinytext NOT NULL,\n\t\t\t\t\tPRIMARY KEY  (id)\n\t\t\t) ENGINE=InnoDB DEFAULT CHARSET=utf8;";
            dbDelta( $sql_log );
            $redirects_table_name = $wpdb->prefix . 'ddp_redirects';
            $sql_redirects = "CREATE TABLE {$redirects_table_name} (\n\t\t\t\t\tid bigint(20) NOT NULL AUTO_INCREMENT,\n\t\t\t\t\tdatime timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,\n\t\t\t\t\tinurl varchar(1024) NOT NULL,\n\t\t\t\t\ttargeturl varchar(1024) NOT NULL,\n\t\t\t\t\thttpcode varchar(3) DEFAULT NULL,\n\t\t\t\t\tPRIMARY KEY  (id)\n\t\t\t) ENGINE=InnoDB DEFAULT CHARSET=utf8;";
            dbDelta( $sql_redirects );
            // Additional plugin setup
            $options = self::get_options();
            self::save_options( $options );
            wp_clear_scheduled_hook( 'ddp_cron' );
            self::log( __( 'Plugin activated.', 'delete-duplicate-posts' ) );
        }
        
        /**
         * Install routines - create database and default options
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   mixed   $network_wide
         * @return  void
         */
        public static function install( $network_wide )
        {
            global  $wpdb ;
            require_once ABSPATH . '/wp-admin/includes/upgrade.php';
            
            if ( is_multisite() && $network_wide ) {
                // Get all blogs in the network and activate plugin on each one
                $blog_ids = $wpdb->get_col( "SELECT blog_id FROM {$wpdb->blogs}" );
                foreach ( $blog_ids as $blog_id ) {
                    switch_to_blog( $blog_id );
                    self::create_table();
                    restore_current_blog();
                }
            } else {
                self::create_table();
            }
        
        }
        
        /**
         * Creating table when a new blog is created
         * https://sudarmuthu.com/blog/how-to-properly-create-tables-in-wordpress-multisite-plugins/
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @version v1.0.1  Sunday, May 14th, 2023.
         * @access  public static
         * @param   mixed   $new_site
         * @return  void
         */
        public static function on_create_blog( $new_site )
        {
            
            if ( is_plugin_active_for_network( 'delete-duplicate-posts/delete-duplicate-posts.php' ) ) {
                switch_to_blog( $new_site->blog_id );
                self::create_table();
                restore_current_blog();
            }
        
        }
        
        /**
         * Deleting the table whenever a blog is deleted
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   mixed   $tables
         * @return  mixed
         */
        public static function on_delete_blog( $tables )
        {
            global  $wpdb, $ddp_fs ;
            $tables[] = $wpdb->prefix . 'ddp_log';
            if ( $ddp_fs->is__premium_only() ) {
                if ( $ddp_fs->can_use_premium_code() ) {
                    // Add redirects table to list of tables to be removed
                    $tables[] = $wpdb->prefix . 'ddp_redirects';
                }
            }
            return $tables;
        }
        
        /**
         * Saves options
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   mixed   $newoptions
         * @return  mixed
         */
        public static function save_options( $newoptions )
        {
            return update_option( 'delete_duplicate_posts_options_v4', $newoptions );
        }
        
        /**
         * Adds link to menu under Tools
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @return  void
         */
        public static function admin_menu_link()
        {
            // only for admins
            if ( !current_user_can( 'manage_options' ) ) {
                return;
            }
            add_management_page(
                'Delete Duplicate Posts',
                'Delete Duplicate Posts',
                'edit_posts',
                'delete-duplicate-posts',
                array( __NAMESPACE__ . '\\Delete_Duplicate_Posts', 'admin_options_page' ),
                41
            );
            add_filter(
                'plugin_action_links_' . plugin_basename( __FILE__ ),
                array( __NAMESPACE__ . '\\Delete_Duplicate_Posts', 'filter_plugin_actions' ),
                10,
                2
            );
        }
        
        /**
         * filter_plugin_actions.
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @param   mixed   $links
         * @param   mixed   $file
         * @return  mixed
         */
        public static function filter_plugin_actions( $links, $file )
        {
            $settings_link = '<a href="tools.php?page=delete-duplicate-posts">' . __( 'Settings', 'delete-duplicate-posts' ) . '</a>';
            array_unshift( $links, $settings_link );
            // before other links
            return $links;
        }
        
        /**
         * Adds help content to plugin page
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @access  public static
         * @return  void
         */
        public static function set_custom_help_content()
        {
            $screen = get_current_screen();
            if ( 'tools_page_delete-duplicate-posts' === $screen->id ) {
                $screen->add_help_tab( array(
                    'id'      => 'ddp_help',
                    'title'   => __( 'Usage and FAQ', 'delete-duplicate-posts' ),
                    'content' => '<h4>' . __( 'What does this plugin do?', 'delete-duplicate-posts' ) . '</h4><p>' . __( 'Helps you clean duplicate posts from your blog. The plugin checks for blogposts on your blog with the same title.', 'delete-duplicate-posts' ) . '</p><p>' . __( "It can run automatically via WordPress's own internal CRON-system, or you can run it automatically.", 'delete-duplicate-posts' ) . '</p><p>' . __( 'It also has a nice feature that can send you an e-mail when Delete Duplicate Posts finds and deletes something (if you have turned on the CRON feature).', 'delete-duplicate-posts' ) . '</p><h4>' . __( 'Help! Something was deleted that was not supposed to be deleted!', 'delete-duplicate-posts' ) . '</h4><p>' . __( 'I am sorry for that, I can only recommend you restore the database you took just before you ran this plugin.', 'delete-duplicate-posts' ) . '</p><p>' . __( 'If you run this plugin, manually or automatically, it is at your OWN risk!', 'delete-duplicate-posts' ) . '</p><p>' . __( 'We have done our best to avoid deleting something that should not be deleted, but if it happens, there is nothing we can do to help you.', 'delete-duplicate-posts' ) . "</p><p><a href='https://cleverplugins.com' target='_blank'>cleverplugins.com</a>.</p>",
                ) );
            }
        }
        
        /**
         * admin_options_page.
         *
         * @author  Lars Koudal
         * @since   v0.0.1
         * @version v1.0.0  Thursday, June 9th, 2022.
         * @version v1.0.1  Thursday, June 9th, 2022.
         * @access  public static
         * @return  void
         */
        public static function admin_options_page()
        {
            global  $ddp_fs, $wpdb ;
            // DELETE NOW
            if ( isset( $_POST['deleteduplicateposts_delete'] ) && isset( $_POST['_wpnonce'] ) ) {
                
                if ( wp_verify_nonce( $_POST['_wpnonce'], 'ddp-clean-now' ) ) {
                    self::cleandupes( 1 );
                    // use the value 1 to indicate it is being run manually.
                }
            
            }
            // RUN NOW!!
            if ( isset( $_POST['ddp_runnow'] ) ) {
                if ( !wp_verify_nonce( $_POST['_wpnonce'], 'ddp-update-options' ) ) {
                    die( esc_html( __( 'Whoops! Some error occured, try again, please!', 'delete-duplicate-posts' ) ) );
                }
            }
            // SAVING OPTIONS
            
            if ( isset( $_POST['delete_duplicate_posts_save'] ) ) {
                if ( !wp_verify_nonce( $_POST['_wpnonce'], 'ddp-update-options' ) ) {
                    die( esc_html( __( 'Whoops! There was a problem with the data you posted. Please go back and try again.', 'delete-duplicate-posts' ) ) );
                }
                $posttypes = array();
                
                if ( isset( $_POST['ddp_pts'] ) ) {
                    $option_array = $_POST['ddp_pts'];
                    $option_count = count( $option_array );
                    for ( $i = 0 ;  $i < $option_count ;  $i++ ) {
                        $posttypes[] = sanitize_text_field( $option_array[$i] );
                    }
                }
                
                
                if ( isset( $_POST['ddp_enabled'] ) ) {
                    $options['ddp_enabled'] = ( 'on' === $_POST['ddp_enabled'] ? true : false );
                } else {
                    $options['ddp_enabled'] = false;
                }
                
                $options['ddp_statusmail'] = ( isset( $_POST['ddp_statusmail'] ) && 'on' === $_POST['ddp_statusmail'] ? true : false );
                $options['ddp_debug'] = ( isset( $_POST['ddp_debug'] ) && 'on' === $_POST['ddp_debug'] ? true : false );
                if ( isset( $_POST['ddp_statusmail_recipient'] ) ) {
                    $options['ddp_statusmail_recipient'] = sanitize_text_field( $_POST['ddp_statusmail_recipient'] );
                }
                if ( isset( $_POST['ddp_schedule'] ) ) {
                    $options['ddp_schedule'] = sanitize_text_field( $_POST['ddp_schedule'] );
                }
                if ( isset( $_POST['ddp_keep'] ) ) {
                    $options['ddp_keep'] = sanitize_text_field( $_POST['ddp_keep'] );
                }
                if ( isset( $_POST['ddp_method'] ) ) {
                    $options['ddp_method'] = sanitize_text_field( $_POST['ddp_method'] );
                }
                if ( isset( $_POST['ddp_resultslimit'] ) ) {
                    $options['ddp_resultslimit'] = sanitize_text_field( $_POST['ddp_resultslimit'] );
                }
                // 301 redirects
                
                if ( isset( $_POST['ddp_redirects'] ) ) {
                    $options['ddp_redirects'] = ( 'on' === $_POST['ddp_redirects'] ? true : false );
                } else {
                    $options['ddp_redirects'] = false;
                }
                
                if ( $ddp_fs->is__premium_only() ) {
                    
                    if ( $ddp_fs->can_use_premium_code() ) {
                        if ( isset( $_POST['ddp_compare_metatag'] ) ) {
                            $options['ddp_compare_metatag'] = sanitize_text_field( $_POST['ddp_compare_metatag'] );
                        }
                        $poststati = array();
                        
                        if ( isset( $_POST['ddp_pstati'] ) ) {
                            $option_array = $_POST['ddp_pstati'];
                            $option_count = count( $option_array );
                            for ( $i = 0 ;  $i < $option_count ;  $i++ ) {
                                $poststati[] = sanitize_text_field( $option_array[$i] );
                            }
                            $options['ddp_pstati'] = $poststati;
                        }
                    
                    }
                
                }
                $options['ddp_pts'] = $posttypes;
                // Previously sanitized
                if ( isset( $_POST['ddp_limit'] ) ) {
                    $options['ddp_limit'] = sanitize_text_field( $_POST['ddp_limit'] );
                }
                self::save_options( $options );
                
                if ( isset( self::$options['ddp_enabled'] ) ) {
                    wp_clear_scheduled_hook( 'ddp_cron' );
                    $interval = self::$options['ddp_schedule'];
                    if ( !$interval ) {
                        $interval = 'hourly';
                    }
                    $nextscheduled = wp_next_scheduled( 'ddp_cron' );
                    if ( !$nextscheduled ) {
                        wp_schedule_event( time(), $interval, 'ddp_cron' );
                    }
                }
                
                echo  '<div class="notice notice-success is-dismissible"><p>' . esc_html( __( 'Settings saved.', 'delete-duplicate-posts' ) ) . '</p></div>' ;
            }
            
            // CLEARING THE LOG
            
            if ( isset( $_POST['ddp_clearlog'] ) ) {
                if ( !wp_verify_nonce( $_POST['_wpnonce'], 'ddp_clearlog_nonce' ) ) {
                    die( esc_html( __( 'Whoops! Some error occured, try again, please!', 'delete-duplicate-posts' ) ) );
                }
                $table_name_log = $wpdb->prefix . 'ddp_log';
                $wpdb->query( "TRUNCATE {$table_name_log};" );
                //phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
                echo  '<div class="updated"><p>' . esc_html( __( 'The log was cleared.', 'delete-duplicate-posts' ) ) . '</p></div>' ;
            }
            
            // REACTIVATE THE DATABASE
            
            if ( isset( $_POST['ddp_reactivate'] ) ) {
                if ( !wp_verify_nonce( $_POST['_wpnonce'], 'ddp_reactivate_nonce' ) ) {
                    die( esc_html( __( 'Whoops! Some error occured, try again, please!', 'delete-duplicate-posts' ) ) );
                }
                self::install( false );
                self::log( 'Reinstalled databases' );
            }
            
            $table_name = $wpdb->prefix . 'posts';
            $pluginfo = get_plugin_data( __FILE__ );
            $version = $pluginfo['Version'];
            $name = $pluginfo['Name'];
            $options = self::get_options();
            ?>






			<?php 
            $css_classes = ' free';
            if ( $ddp_fs->is__premium_only() ) {
                if ( $ddp_fs->can_use_premium_code() ) {
                    $css_classes = ' premium_ver';
                }
            }
            ?>

			<div class="wrap<?php 
            echo  esc_attr( $css_classes ) ;
            ?>">
				<h2>Delete Duplicate Posts <span>v. <?php 
            echo  esc_html( self::get_plugin_version() ) ;
            ?></span></h2>

				<div class="ddp_content_wrapper">
					<div class="ddp_content_cell">


						<div id="delete-duplicate-posts-tabs">
							<ul>
								<li><a href="#duplicates-tab">Duplicates</a></li>
								<li><a href="#log-tab">Log</a></li>
								<li><a href="#settings-tab">Settings</a></li>
								<li class="pro"><a href="#redirects-tab">Redirects</a></li>
							</ul>
							<div id="duplicates-tab">
								<div id="ddp-dashboard">
									<?php 
            
            if ( $options['ddp_enabled'] ) {
                $interval = $options['ddp_schedule'];
                if ( !$interval ) {
                    $interval = 'hourly';
                }
                $nextscheduled = wp_next_scheduled( 'ddp_cron' );
                
                if ( !$nextscheduled ) {
                    // plugin active, but the cron needs to be activated also..
                    $options['last_interval'] = $interval;
                    self::save_options( $options );
                    wp_schedule_event( time(), $interval, 'ddp_cron' );
                    //}
                }
            
            } else {
                wp_unschedule_hook( 'ddp_cron' );
            }
            
            $totaldeleted = get_option( 'ddp_deleted_duplicates' );
            ?>
									<div class="statusdiv">
										<div class="statusmessage"></div>
										<div class="dupelist">
											<div id="requestTime"></div>
											<table id="ddp_dupetable" class="wp-list-table widefat fixed striped table-view-list"></table>
										</div>

									</div>

								</div><!-- #dashboard -->





							</div>


							<div id="log-tab">

								<div id="log">
									<h3><?php 
            esc_html_e( 'The Log', 'delete-duplicate-posts' );
            ?></h3>
									<div class="spinner is-active"></div>
									<ul class="large-text" name="ddp_log" id="ddp_log"></ul>
								</div>
								<p>
								<form method="post" id="ddp_clearlog">
									<?php 
            wp_nonce_field( 'ddp_clearlog_nonce' );
            ?>
									<input class="button-secondary" type="submit" name="ddp_clearlog" value="<?php 
            esc_html_e( 'Reset log', 'delete-duplicate-posts' );
            ?>" />
								</form>
								</p>





							</div>

							<div id="redirects-tab" class="pro">
								<h3>Redirects</h3>

								<?php 
            $output = '<p>Redirects is a feature in <a href="https://cleverplugins.com/delete-duplicate-posts/" target="_blank" rel="noopener">the premium version</a></p>';
            if ( $ddp_fs->is__premium_only() ) {
                if ( $ddp_fs->can_use_premium_code() ) {
                    $output = '';
                }
            }
            echo  wp_kses( $output, array(
                'p' => array(),
                'a' => array(
                'href'   => array(),
                'target' => array(),
                'rel'    => array(),
            ),
            ) ) ;
            if ( $ddp_fs->is__premium_only() ) {
                if ( $ddp_fs->can_use_premium_code() ) {
                    ?>
										<table id="ddp_redirtable" class="wp-list-table widefat fixed striped table-view-list"></table>
										<?php 
                }
            }
            ?>
								
							</div>




							<div id="settings-tab">
								<div id="ddp-configuration">
									<h3><?php 
            esc_html_e( 'Settings', 'delete-duplicate-posts' );
            ?></h3>
									<p>
										<?php 
            $nextscheduled = wp_next_scheduled( 'ddp_cron' );
            
            if ( $nextscheduled ) {
                ?>
									<div class="notice notice-info is-dismissible">
										<h3><span class="dashicons dashicons-saved"></span> Automatically Deleting Duplicates</h3>
										<?php 
                echo  '<p class="cronstatus center">' . esc_html__( 'You have enabled automatic deletion, so I am running on automatic. I will take care of everything...', 'delete-duplicate-posts' ) . '</p>' ;
                echo  '<p class="center">' ;
                echo  sprintf(
                    // translators: Showing when the next check happens and what the current time is
                    esc_html( __( 'Next automated check %1$s. Current time %2$s', 'delete-duplicate-posts' ) ),
                    '<strong>' . esc_html( date_i18n( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), $nextscheduled ) ) . '</strong>',
                    '<strong>' . esc_html( date_i18n( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), time() ) ) . '</strong>'
                ) ;
                echo  '</p>' ;
                ?>
									</div>
								<?php 
            }
            
            ?>
								</p>
								<form method="post" id="delete_duplicate_posts_options">
									<?php 
            wp_nonce_field( 'ddp-update-options' );
            ?>
									<table width="100%" cellspacing="2" cellpadding="5" class="form-table">


										<tr valign="top">
											<th><label for="ddp_pts"><?php 
            esc_html_e( 'Which post types?:', 'delete-duplicate-posts' );
            ?></label>
											</th>
											<td>
												<?php 
            $builtin = array( 'post', 'page', 'attachment' );
            $args = array(
                'public'   => true,
                '_builtin' => false,
            );
            $output = 'names';
            $operator = 'and';
            $post_types = get_post_types( $args, $output, $operator );
            $post_types = array_merge( $builtin, $post_types );
            $checked_post_types = $options['ddp_pts'];
            
            if ( $post_types ) {
                ?>
													<ul class="radio">
														<?php 
                $step = 0;
                if ( !is_array( $checked_post_types ) ) {
                    $checked_post_types = array();
                }
                foreach ( $post_types as $pt ) {
                    $checked = array_search( $pt, $checked_post_types, true );
                    ?>
															<li><input type="checkbox" name="ddp_pts[]" id="ddp_pt-<?php 
                    echo  esc_attr( $step ) ;
                    ?>" value="<?php 
                    echo  esc_html( $pt ) ;
                    ?>" <?php 
                    if ( false !== $checked ) {
                        echo  ' checked' ;
                    }
                    ?> />
																<label for="ddp_pt-<?php 
                    echo  esc_attr( $step ) ;
                    ?>"><?php 
                    echo  esc_html( $pt ) ;
                    ?></label>
																<?php 
                    // Count for each post type
                    $postinfo = wp_count_posts( $pt );
                    $othercount = 0;
                    foreach ( $postinfo as $pi ) {
                        $othercount = $othercount + intval( $pi );
                    }
                    // translators: Total number of deleted duplicates
                    echo  '<small>' . sprintf( esc_html__( '(%s total found)', 'delete-duplicate-posts' ), esc_html( number_format_i18n( $othercount ) ) ) . '</small>' ;
                    ?>
															</li>
														<?php 
                    ++$step;
                }
                ?>
													</ul>
												<?php 
            }
            
            ?>
												<p class="description">
													<?php 
            esc_html_e( 'Choose which post types to scan for duplicates.', 'delete-duplicate-posts' );
            ?>
												</p>
											</td>
										</tr>

										<tr>
											<th><label for="ddp_pstati"><?php 
            esc_html_e( 'Post status', 'delete-duplicate-posts' );
            ?></label>
											</th>
											<td>
												<?php 
            $stati = array(
                'publish' => (object) array(
                'label'                     => 'Published',
                'show_in_admin_status_list' => true,
            ),
            );
            if ( $ddp_fs->is__premium_only() ) {
                if ( $ddp_fs->can_use_premium_code() ) {
                    $stati = get_post_stati( array(), 'objects' );
                }
            }
            $checked_post_stati = $options['ddp_pstati'];
            
            if ( $stati ) {
                ?>
													<ul class="checkbox">
														<?php 
                $staticount = count( $stati );
                foreach ( $stati as $key => $st ) {
                    
                    if ( $st->show_in_admin_status_list ) {
                        $checked = array_search( $key, $checked_post_stati, true );
                        ?>
																<li>
																	<input type="checkbox" name="ddp_pstati[]" id="ddp_pstatus-<?php 
                        echo  esc_attr( $key ) ;
                        ?>" value="<?php 
                        echo  esc_attr( $key ) ;
                        ?>" <?php 
                        if ( false !== $checked ) {
                            echo  ' checked' ;
                        }
                        if ( 1 === $staticount ) {
                            echo  ' disabled' ;
                        }
                        ?> /><label for="ddp_pstatus-<?php 
                        echo  esc_attr( $key ) ;
                        ?>">
																		<?php 
                        echo  esc_html( $key . ' (' . $st->label . ')' ) ;
                        if ( 'trash' === $key ) {
                            echo  ' <small>Warning, enabling this can give false results. Only enable if you know what you are doing.</small>' ;
                        }
                        ?>
																	</label>
																</li>
														<?php 
                        ++$step;
                    }
                
                }
                ?>
													</ul>
												<?php 
            }
            
            ?>
											</td>


										</tr>
										<?php 
            $comparemethod = 'titlecompare';
            global  $ddp_fs ;
            if ( $ddp_fs->is__premium_only() ) {
                if ( $ddp_fs->can_use_premium_code() ) {
                    
                    if ( isset( $options['ddp_method'] ) ) {
                        $comparemethod = $options['ddp_method'];
                    } else {
                        $comparemethod = 'titlecompare';
                    }
                
                }
            }
            ?>
										<tr valign="top">
											<th><?php 
            esc_html_e( 'Comparison Method', 'delete-duplicate-posts' );
            ?></th>
											<td>
												<ul class="ddpcomparemethod">

													<li>
														<label>
															<input type="radio" name="ddp_method" value="titlecompare" <?php 
            checked( 'titlecompare', $comparemethod );
            ?> />
															<?php 
            esc_html_e( 'Compare by title (default)', 'delete-duplicate-posts' );
            ?>
															<span class="optiondesc"><?php 
            esc_html_e( 'Looks at the title of the post itself.', 'delete-duplicate-posts' );
            ?></span>
														</label>

													</li>

													<?php 
            global  $ddp_fs ;
            if ( $ddp_fs->is__premium_only() ) {
                
                if ( $ddp_fs->can_use_premium_code() ) {
                    ?>
															<li>
																<label>
																	<input type="radio" name="ddp_method" value="metacompare" <?php 
                    checked( 'metacompare', $comparemethod );
                    ?> />
																	<?php 
                    esc_html_e( 'Compare by meta tag', 'delete-duplicate-posts' );
                    ?>
																	<span class="optiondesc"><?php 
                    esc_html_e( 'Compare by any meta tag.', 'delete-duplicate-posts' );
                    ?></span>
																</label>
																<?php 
                    $metavalues = $wpdb->get_results( "SELECT DISTINCT meta_key FROM {$wpdb->postmeta} ORDER by meta_key;", ARRAY_A );
                    ?>
																<div class="ddp-compare-details">

																	<select name="ddp_compare_metatag" id="ddp_compare_metatag">
																		<?php 
                    
                    if ( is_array( $metavalues ) ) {
                        $selectedmeta = false;
                        if ( isset( $options['ddp_compare_metatag'] ) ) {
                            $selectedmeta = $options['ddp_compare_metatag'];
                        }
                        
                        if ( !$selectedmeta ) {
                            $selectedmeta = '';
                            $options['ddp_compare_metatag'] = '';
                        }
                        
                        foreach ( $metavalues as $mv ) {
                            ?>
																				<option value="<?php 
                            echo  esc_attr( $mv['meta_key'] ) ;
                            ?>" <?php 
                            selected( esc_attr( $mv['meta_key'] ), $options['ddp_compare_metatag'] );
                            ?>>
																					<?php 
                            echo  esc_attr( $mv['meta_key'] ) ;
                            ?></option>
																		<?php 
                        }
                    }
                    
                    ?>
																	</select>
																</div>

															</li>
													<?php 
                }
            
            }
            ?>
												</ul>
											</td>
										</tr>
										<tr>
											<th><label for="ddp_keep"><?php 
            esc_html_e( 'Delete which posts?:', 'delete-duplicate-posts' );
            ?></label></th>
											<td>

												<select name="ddp_keep" id="ddp_keep">
													<option value="oldest" <?php 
            if ( 'oldest' === $options['ddp_keep'] ) {
                echo  'selected="selected"' ;
            }
            ?>><?php 
            esc_html_e( 'Keep oldest', 'delete-duplicate-posts' );
            ?></option>
													<option value="latest" <?php 
            if ( 'latest' === $options['ddp_keep'] ) {
                echo  'selected="selected"' ;
            }
            ?>><?php 
            esc_html_e( 'Keep latest', 'delete-duplicate-posts' );
            ?></option>
												</select>
												<p class="description">
													<?php 
            esc_html_e( 'Keep the oldest or the latest version of duplicates? Default is keeping the oldest, and deleting any subsequent duplicate posts', 'delete-duplicate-posts' );
            ?>
												</p>
											</td>
										</tr>








										<?php 
            if ( $ddp_fs->is__premium_only() ) {
                
                if ( $ddp_fs->can_use_premium_code() ) {
                    ?>
												<tr valign="top">
													<th><?php 
                    esc_html_e( 'Enable 301 redirects?:', 'delete-duplicate-posts' );
                    ?>

													</th>
													<td><label for="ddp_redirects">
															<input type="checkbox" id="ddp_redirects" name="ddp_redirects" <?php 
                    if ( true === $options['ddp_redirects'] ) {
                        echo  'checked="checked"' ;
                    }
                    ?>>
															<p class="description">
																<?php 
                    esc_html_e( 'Automatically 301 redirects deleted posts to the original.', 'delete-duplicate-posts' );
                    ?></p>
															<?php 
                    ?>
														</label>
													</td>
												</tr>
										<?php 
                }
            
            }
            ?>
										<tr>
											<td colspan="2">
												<hr>
												<h3><?php 
            _e( 'Delete Duplicates Automatically', 'security-ninja' );
            ?></h3>
											</td>
										</tr>

										<tr valign="top">
											<th><?php 
            esc_html_e( 'Enable automatic deletion?:', 'delete-duplicate-posts' );
            ?>
											</th>
											<td><label for="ddp_enabled">
													<input type="checkbox" id="ddp_enabled" name="ddp_enabled" <?php 
            if ( true === $options['ddp_enabled'] ) {
                echo  'checked="checked"' ;
            }
            ?>>
													<p class="description">
														<?php 
            esc_html_e( 'Clean duplicates automatically.', 'delete-duplicate-posts' );
            ?></p>
												</label>
											</td>
										</tr>

										<tr>
											<th><label for="ddp_resultslimit"><?php 
            esc_html_e( 'How many:', 'delete-duplicate-posts' );
            ?></label>
											</th>
											<td>

												<?php 
            $dupe_options = array(
                0     => __( 'No limit', 'delete-duplicate-posts' ),
                10000 => number_format_i18n( '10000' ),
                5000  => number_format_i18n( '5000' ),
                2500  => number_format_i18n( '2500' ),
                1000  => number_format_i18n( '1000' ),
                500   => '500',
                250   => '250',
                100   => '100',
                50    => '50',
                10    => '10',
            );
            ?>
												<select name="ddp_resultslimit" id="ddp_resultslimit">
													<?php 
            foreach ( $dupe_options as $key => $label ) {
                ?>
														<option value="<?php 
                echo  esc_attr( $key ) ;
                ?>" <?php 
                selected( $options['ddp_resultslimit'], $key );
                ?>>
															<?php 
                echo  esc_attr( $label ) ;
                ?></option>
													<?php 
            }
            ?>
												</select>

												<p class="description">
													<?php 
            esc_html_e( 'If you have many duplicates, the plugin might time out before finding them all. Try limiting the amount of duplicates here. Default: Unlimited.', 'delete-duplicate-posts' );
            ?><br>
													<strong><?php 
            esc_html_e( 'This only applies to automatic (CRON) jobs.', 'delete-duplicate-posts' );
            ?></strong>
												</p>
											</td>
										</tr>

										<tr>
											<th><label for="ddp_schedule"><?php 
            esc_html_e( 'How often?:', 'delete-duplicate-posts' );
            ?></label>
											</th>
											<td>

												<select name="ddp_schedule" id="ddp_schedule">
													<?php 
            $schedules = wp_get_schedules();
            if ( $schedules ) {
                foreach ( $schedules as $key => $sch ) {
                    ?>
															<option value="<?php 
                    echo  esc_attr( $key ) ;
                    ?>" <?php 
                    if ( isset( $options['ddp_schedule'] ) && esc_attr( $key ) === $options['ddp_schedule'] ) {
                        echo  esc_html( 'selected="selected"' ) ;
                    }
                    ?>><?php 
                    echo  esc_html( $sch['display'] ) ;
                    ?></option>
													<?php 
                }
            }
            ?>
												</select>
												<p class="description">
													<?php 
            esc_html_e( 'How often should the cron job run?', 'delete-duplicate-posts' );
            ?></p>
											</td>
										</tr>
										<tr>
											<td colspan="2">
												<hr>
											</td>
										</tr>

										<tr>
											<th><?php 
            esc_html_e( 'Send status mail?:', 'delete-duplicate-posts' );
            ?></th>
											<td>
												<label for="ddp_statusmail">
													<input type="checkbox" id="ddp_statusmail" name="ddp_statusmail" <?php 
            if ( isset( $options['ddp_statusmail'] ) && true === $options['ddp_statusmail'] ) {
                echo  'checked="checked"' ;
            }
            ?>>
													<p class="description">
														<?php 
            esc_html_e( 'Sends a status email if duplicates have been found.', 'delete-duplicate-posts' );
            ?>
													</p>
												</label>
											</td>
										</tr>

										<tr>
											<th><?php 
            esc_html_e( 'Email recipient:', 'delete-duplicate-posts' );
            ?></th>
											<td>
												<label for="ddp_statusmail_recipient">

													<input type="text" class="regular-text" id="ddp_statusmail_recipient" name="ddp_statusmail_recipient" value="<?php 
            echo  esc_html( $options['ddp_statusmail_recipient'] ) ;
            ?>">
													<p class="description">
														<?php 
            esc_html_e( 'Who should get the notification email.', 'delete-duplicate-posts' );
            ?></p>
												</label>
											</td>
										</tr>



										<tr>
											<td colspan="2">
												<hr>
											</td>
										</tr>

										<tr>
											<th><?php 
            esc_html_e( 'Enable debug logging?:', 'delete-duplicate-posts' );
            ?></th>
											<td>
												<label for="ddp_debug">
													<input type="checkbox" id="ddp_debug" name="ddp_debug" <?php 
            if ( isset( $options['ddp_debug'] ) && true === $options['ddp_debug'] ) {
                echo  'checked="checked"' ;
            }
            ?>>
													<p class="description">
														<?php 
            esc_html_e( 'Should only be enabled if debugging a problem.', 'delete-duplicate-posts' );
            ?>
													</p>
												</label>
											</td>
										</tr>
										<th colspan=2><input type="submit" class="button-primary" name="delete_duplicate_posts_save" value="<?php 
            esc_html_e( 'Save Settings', 'delete-duplicate-posts' );
            ?>" /></th>
										</tr>
									</table>
								</form>
								</div><!-- #configuration -->



							</div>
						</div>
						<script type="text/javascript">
							jQuery(document).ready(function($) {
								$('#delete-duplicate-posts-tabs').tabs();
							});
						</script>


					</div>

					<?php 
            include_once 'sidebar.php';
            
            if ( function_exists( 'ddp_fs' ) ) {
                global  $ddp_fs ;
                if ( $ddp_fs->is__premium_only() ) {
                    
                    if ( $ddp_fs->can_use_premium_code() ) {
                        $beacon_script = '<script type="text/javascript">!function(e,t,n){function a(){var e=t.getElementsByTagName("script")[0],n=t.createElement("script");n.type="text/javascript",n.async=!0,n.src="https://beacon-v2.helpscout.net",e.parentNode.insertBefore(n,e)}if(e.Beacon=n=function(t,n,a){e.Beacon.readyQueue.push({method:t,options:n,data:a})},n.readyQueue=[],"complete"===t.readyState)return a();e.attachEvent?e.attachEvent("onload",a):e.addEventListener("load",a,!1)}(window,document,window.Beacon||function(){});</script><script type="text/javascript">window.Beacon("init", "3ad4aee5-9cb0-4ad7-8f06-0002958ad6ab")</script>';
                        echo  wp_kses( $beacon_script, array(
                            'script' => array(
                            'type'  => array(),
                            'src'   => array(),
                            'async' => array(),
                        ),
                        ) ) ;
                    }
                
                }
            }
            
            ?>
				</div>

			</div>












<?php 
        }
    
    }
    //End Class
}

if ( class_exists( __NAMESPACE__ . '\\Delete_Duplicate_Posts' ) ) {
    $delete_duplicate_posts_var = new Delete_Duplicate_Posts();
}